import datetime
from unittest.mock import patch
import pytest
import random

from fortunecookiepkg.fortunecookie import (
    FORTUNES,
    get_random_fortune,
    get_lucky_numbers,
    get_daily_fortune,
    get_custom_fortune,
    get_themed_fortune,
)

class Tests:
    #Fixtures
    @pytest.fixture
    def example_fixture(self):
        """
        An example fixture used for setup/teardown before/after tests are run.
        """
        yield

    #Test functions
    def test_sanity_check(self, example_fixture):
        """
        Test debugging is working with a simple check.
        """
        expected = True
        actual = True
        assert actual==expected, "Expected True to equal True."
        
    #Random fortune tests
    def test_get_random_fortune_randomness(self):
        """
        Given multiple calls, two outputs should not always be the same, ensuring randomness. 
        """
        res1 = get_random_fortune()
        res2 = get_random_fortune()
        assert res1 != res2, f"Expected get_random_fortune() to return 2 different fortunes, but got identical results: '{res1}'."

    #Lucky number tests
    def test_get_lucky_numbers_length_valid(self):
        """
        If the input is valid, the array should be the same length as the given input"
        """
        valid_len = random.randint(1, 10)
        res = get_lucky_numbers(valid_len)
        assert len(res) == valid_len, f"Expected get_lucky_numbers() to return {valid_len} numbers. Instead, it returned an array with {len(res)} numbers"

    def test_get_lucky_numbers_length_invalid(self):
        """
        If there is invalid input, the array should have length of 0.
        """
        invalid_len = random.randint(11, 100)
        res = get_lucky_numbers(invalid_len)
        assert res == 0, f"Expected get_lucky_numbers() to return 0 numbers. Instead, it returned an array with {len(res)} numbers"  

    def test_get_lucky_numbers_randomness(self):
        """
        Given the same length, two outputs should not be the same which ensures randomness.
        """
        valid_len = random.randint(1, 10)
        res1 = get_lucky_numbers(valid_len)
        res2 = get_lucky_numbers(valid_len)
        assert res1 != res2, f"Expected get_lucky_numbers() to return 2 different arrays, given the same length. Instead, it returned two arrays with identical values"

    #Custom fortune tests
    def test_get_custom_fortune_name(self):
        """
        Output should include the given name.
        """
        name = "Sam"
        res = get_custom_fortune(name)
        assert name in res, f"Expected '{name}' in the output but got '{res}'"
   
    def test_get_custom_fortune_empty_name(self):
        """
        If the name is empty, the function should still return a valid fortune.
        """
        res = get_custom_fortune("")
        assert isinstance(res, str), "Expected output to be a string"
        assert res.strip() != "", "Expected a non-empty fortune, but got an empty string"
   
    def test_get_custom_fortune_long_name(self):
        """
        If system has length limits or unexpected behavior with long strings, function should still work.
        """

        name = "A" * 100
        res = get_custom_fortune(name)
        assert name in res, f"Expected the long name in the output but got '{res}'"
    
    def test_get_custom_fortune_special_chars(self):
        """
        Function should corrrectly handle names with special characters in them.
        """
        name = "@l!c3_42"
        res = get_custom_fortune(name)
        assert name in res, f"Expected '{name}' in the output but got '{res}'"
    
    #Themed fortune tests
    def test_get_themed_fortune_invalid_theme(self):
        """ 
        Provided theme should be valid.
        """
        res = get_themed_fortune("unknown")
        assert "There are no fortunes for your provided theme" in res, f"Expected an error message but got '{res}'"

    def test_get_themed_fortune_empty(self):
        """
        If an empty string is given as a theme, it should return an error message.
        """
        res = get_themed_fortune("")
        assert "There are no fortunes for your provided theme" in res, f"Expected an error message but got '{res}'"

    @pytest.mark.parametrize("theme", [" love ", "  career  ", " happiness "])
    def test_get_themed_fortune_spaces(self, theme):
        """
        If there are spaces, hand the extra spaces. 
        """
        res = get_themed_fortune(theme.strip())
        assert isinstance(res, str), "Expected output to be a string"
        assert len(res) > 0, f"Expected a non-empty fortune but got '{res}'"

    #Daily fortune tests
    def test_get_daily_fortune_exists(self):
        """
        Tests that the fortune for a given day exists in the FORTUNES array.
        """
        fortune = get_daily_fortune('wednesday')
        assert fortune in FORTUNES

    def test_get_daily_fortune_is_accurate(self):
        """
        Tests that the fortune for a given day of the week remains consistent given different cases.
        """
        fortune1 = get_daily_fortune('monday')
        fortune2 = get_daily_fortune('Monday')
        assert (fortune1 == fortune2)

    def test_get_daily_fortune_returns_string(self):
        """
        Tests that the daily fortune function returns a non-empty string, given a valid day of the week.
        """
        fortune = get_daily_fortune('tuesday')
        assert len(fortune) != 0, f"Expected get_daily_fortune() to return a non-empty string of length > 0. Instead, it returned a string with length {len(fortune)}"

    def test_get_daily_fortune_returns_string(self):
        """    
        Tests that the daily fortune function returns an empty string, given an invalid day of the week
        """
        fortune = get_daily_fortune('invalid_day')
        assert len(fortune) == 0, f"Expected get_daily_fortune() to return string of length 0. Instead, it returned a string with length {len(fortune)}"
    
    @pytest.mark.parametrize("invalid_day", [None, 123, [], {}, set(), 5.6])
    def test_get_daily_fortune_invalid_types(self, invalid_day):
        """
        Ensure the funciton can handle invalid data types.
        """
        fortune = get_daily_fortune(invalid_day)
        assert len(fortune) == 0, f"Expected empty string for invalid input but got '{fortune}'"
  
    @pytest.mark.parametrize("day", ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"])
    def test_get_daily_fortune_valid_days(self, day):
        """
        Ensure function returns a valid fortune for all days of the week
        """
        fortune = get_daily_fortune(day)
        assert isinstance(fortune, str) and len(fortune) > 0, f"Expected a valid fortune but got '{fortune}'"


import random
import datetime

#List of fortunes
FORTUNES = [
    "Do not be afraid of competition.",
    "An exciting opportunity lies ahead of you.",
    "You love peace.",
    "Get your mind set…confidence will lead you on.",
    "You will always be surrounded by true friends.",
    "Sell your ideas—they have exceptional merit.",
    "You should be able to undertake and complete anything.",
    "You are kind and friendly.",
    "You are wise beyond your years.",
    "Your ability to juggle many tasks will take you far.",
    "A routine task will turn into an enchanting adventure.",
    "Beware of all enterprises that require new clothes.",
    "Be true to your work, your word, and your friend.",
    "Goodness is the only investment that never fails.",
    "A journey of a thousand miles begins with a single step.",
    "Forget injuries; never forget kindnesses.",
    "Respect yourself and others will respect you.",
    "A man cannot be comfortable without his own approval.",
    "Always do right. This will gratify some people and astonish the rest.",
    "It is easier to stay out than to get out.",
    "Sing every day and chase the mean blues away.",
    "You will receive money from an unexpected source.",
    "Attitude is a little thing that makes a big difference.",
    "Plan for many pleasures ahead.",
    "Experience is the best teacher.",
    "You will be happy with your spouse.",
    "Expect the unexpected.",
    "Stay healthy. Walk a mile.",
    "The family that plays together stays together.",
    "Eat chocolate to have a sweeter life.",
    "Once you make a decision the universe conspires to make it happen.",
    "Make yourself necessary to someone.",
    "The only way to have a friend is to be one.",
    "Nothing great was ever achieved without enthusiasm.",
    "Dance as if no one is watching.",
    "Live this day as if it were your last.",
    "Your life will be happy and peaceful.",
    "Reach for joy and all else will follow.",
    "Move in the direction of your dreams.",
    "Bloom where you are planted.",
    "Appreciate. Appreciate. Appreciate.",
    "Happy news is on its way.",
    "Paradise is always where love dwells.",
    "The one you love is closer than you think.",
    "Love is like wildflowers…it is often found in the most unlikely places.",
    "In dreams and in love there are no impossibilities.",
    "Love isn't something you find. Love is something that finds you.",
    "True love is not something that comes every day; follow your heart, it knows the right answer."
]

#Categories of themed fortunes
THEMED_FORTUNES = {
    "love": [
        "Paradise is always where love dwells.",
        "The one you love is closer than you think.",
        "Love is like wildflowers…it is often found in the most unlikely places.",
        "In dreams and in love there are no impossibilities.",
        "Love isn't something you find. Love is something that finds you.",
        "True love is not something that comes every day; follow your heart, it knows the right answer."
        "You will be happy with your spouse."], 
    "career": [
        "Do not be afraid of competition.",
        "An exciting opportunity lies ahead of you.",
        "Get your mind set…confidence will lead you on.",
        "Sell your ideas—they have exceptional merit.",
        "You should be able to undertake and complete anything.",
        "Your ability to juggle many tasks will take you far.",
        "A routine task will turn into an enchanting adventure.",
        "Beware of all enterprises that require new clothes.",
        "Be true to your work, your word, and your friend.",
        "Goodness is the only investment that never fails."], 
    "happiness": [
        "Your life will be happy and peaceful.",
        "Reach for joy and all else will follow.",
        "Happy news is on its way.",
        "Dance as if no one is watching.",
        "Live this day as if it were your last.",
        "Bloom where you are planted.",
        "Appreciate. Appreciate. Appreciate."]
}

#Days of the week
week_days = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday']

def get_random_fortune():
    """
    Returns a random fortune from the list
    """
    return random.choice(FORTUNES)

def get_lucky_numbers(length: int):
    """
    Returns a set of len lucky numbers
    """
    len_num = int(length)
    if 1 <= len_num <= 10:
        return random.sample(range(1, 99), len_num)
    else:
        return 0
    
def get_daily_fortune(day:str):
    """
    Returns the same fortune for everyone given a certain day of the week.
    """
    if not isinstance(day, str):
        return ""
    
    if day.lower() in week_days:  
        day_num = week_days.index(day.lower()) + 1  
        today = datetime.date.today().timetuple().tm_yday
        index = int((today%len(FORTUNES)) / day_num)
        return FORTUNES[index]
    else:
        return ""
    
def get_custom_fortune(name:str):
    """
    Returns a personalized fortune.
    """
    if not name.strip():
        return random.choice(FORTUNES)
    return f"{name}, {random.choice(FORTUNES)}"

def get_themed_fortune(theme:str):
    """
    Returns a fortune based on a specific theme
    """
    themes = theme.strip().lower
    if not themes:
        return "Invalid theme. Please choose 'love', 'career', or 'happiness'."
    return random.choice(THEMED_FORTUNES.get(theme.strip().lower(), ["There are no fortunes for your provided theme, please choose 'love', 'career', or 'happiness'"]))




import argparse
from fortunecookiepkg.fortunecookie import (
    get_random_fortune,
    get_lucky_numbers,
    get_daily_fortune,
    get_custom_fortune,
    get_themed_fortune,
)

def main():
    """
    Recieve the fortune or lucky numbers
    """
    parser = argparse.ArgumentParser(description="Get your fortune!")
    parser.add_argument("--random", action="store_true", help="Get a random fortune.")
    parser.add_argument("--numbers", metavar="LEN", help="Get your lucky number(s). Choose how many numbers you want from 1-10 (inclusive).")
    parser.add_argument("--daily", metavar="DAY", help="Get a fortune based on the day of the week.")
    parser.add_argument("--custom", metavar="NAME", help="Get a personalized fortune.")
    parser.add_argument("--themed", metavar="THEME", help="Get a themed fortune. The themes are love, career, and happiness.")

    args = parser.parse_args()

    if args.random:
        print(get_random_fortune())
    elif args.numbers:
        lucky_numbers = get_lucky_numbers(args.numbers)
        len_nums = len(lucky_numbers)
        if len_nums > 0:
            print("Lucky Numbers:", lucky_numbers)
        else:
            parser.print_help()
        
    elif args.daily:
        daily_fortune = get_daily_fortune(args.daily)
        len_daily_fortune = len(daily_fortune)
        if len_daily_fortune > 0:
            print("Today's Fortune:", get_daily_fortune(args.daily))
        else:
            parser.print_help()
    elif args.custom:
        print(get_custom_fortune(args.custom))
    elif args.themed: 
        print(get_themed_fortune(args.themed))
    else:
        parser.print_help()

if __name__ == "__main__":
    #run main function
    main()